@@ -254,7 +254,7 @@ convert_to_one_line (struct buffer *buf)
 void
 buf_write_string_file (const struct buffer *buf, const char *filename, int fd)
 {
-  const int len = strlen (BPTR (buf));
+  const int len = strlen ((char *) BPTR (buf));
   const int size = write (fd, BPTR (buf), len);
   if (size != len)
     msg (M_ERR, "Write error on file '%s'", filename);
@@ -282,10 +282,10 @@ gc_malloc (size_t size, bool clear, struct gc_arena *a)
 #endif
       check_malloc_return (e);
       ret = (char *) e + sizeof (struct gc_entry);
-      //mutex_lock_static (L_GC_MALLOC);
+      /*mutex_lock_static (L_GC_MALLOC);*/
       e->next = a->list;
       a->list = e;
-      //mutex_unlock_static (L_GC_MALLOC);
+      /*mutex_unlock_static (L_GC_MALLOC);*/
     }
   else
     {
@@ -307,10 +307,10 @@ void
 x_gc_free (struct gc_arena *a)
 {
   struct gc_entry *e;
-  //mutex_lock_static (L_GC_MALLOC);
+  /*mutex_lock_static (L_GC_MALLOC);*/
   e = a->list;
   a->list = NULL;
-  //mutex_unlock_static (L_GC_MALLOC);
+  /*mutex_unlock_static (L_GC_MALLOC);*/
   
   while (e != NULL)
     {
@@ -365,7 +365,7 @@ buf_rmtail (struct buffer *buf, uint8_t remove)
 void
 buf_null_terminate (struct buffer *buf)
 {
-  char *last = BLAST (buf);
+  char *last = (char *) BLAST (buf);
   if (last && *last == '\0') /* already terminated? */
     return;
 
@@ -384,7 +384,7 @@ buf_chomp (struct buffer *buf)
 {
   while (true)
     {
-      char *last = BLAST (buf);
+      char *last = (char *) BLAST (buf);
       if (!last)
 	break;
       if (char_class (*last, CC_CRLF|CC_NULL))
@@ -475,9 +475,9 @@ string_alloc_buf (const char *str, struct gc_arena *gc)
   ASSERT (str);
 
 #ifdef DMALLOC
-  buf_set_read (&buf, string_alloc_debug (str, gc, file, line), strlen (str) + 1);
+  buf_set_read (&buf, (uint8_t*) string_alloc_debug (str, gc, file, line), strlen (str) + 1);
 #else
-  buf_set_read (&buf, string_alloc (str, gc), strlen (str) + 1);
+  buf_set_read (&buf, (uint8_t*) string_alloc (str, gc), strlen (str) + 1);
 #endif
 
   if (buf.len > 0) /* Don't count trailing '\0' as part of length */
