@@ -98,14 +98,12 @@ init_remote_list (struct context *c)
     }
 }
 
-void
-context_init_1 (struct context *c)
+/*
+ * Query for private key and auth-user-pass username/passwords
+ */
+static void
+init_query_passwords (struct context *c)
 {
-  context_clear_1 (c);
-
-  packet_id_persist_init (&c->c1.pid_persist);
-  init_remote_list (c);
-
 #if defined(USE_CRYPTO) && defined(USE_SSL)
   /* Certificate password input */
   if (c->options.key_pass_file)
@@ -114,11 +112,22 @@ context_init_1 (struct context *c)
   
 #if P2MP
   /* Auth user/pass input */
-  if (c->options.auth_user_pass_file)
-    {
-      auth_user_pass_setup (c->options.auth_user_pass_file);
-    }
+ {
+   if (c->options.auth_user_pass_file)
+     auth_user_pass_setup (c->options.auth_user_pass_file);
+ }
 #endif
+}
+
+void
+context_init_1 (struct context *c)
+{
+  context_clear_1 (c);
+
+  packet_id_persist_init (&c->c1.pid_persist);
+  init_remote_list (c);
+
+  init_query_passwords (c);
 
 #ifdef ENABLE_HTTP_PROXY
   if (c->options.http_proxy_options)
@@ -372,8 +381,9 @@ static void
 do_uid_gid_chroot (struct context *c, bool no_delay)
 {
   static const char why_not[] = "will be delayed because of --client, --pull, or --up-delay";
+  struct context_0 *c0 = c->c0;
 
-  if (c->first_time && !c->c2.uid_gid_set)
+  if (c->first_time && c0 && !c0->uid_gid_set)
     {
       /* chroot if requested */
       if (c->options.chroot_dir)
@@ -387,11 +397,11 @@ do_uid_gid_chroot (struct context *c, bool no_delay)
       /* set user and/or group that we want to setuid/setgid to */
       if (no_delay)
 	{
-	  set_group (&c->c2.group_state);
-	  set_user (&c->c2.user_state);
-	  c->c2.uid_gid_set = true;
+	  set_group (&c0->group_state);
+	  set_user (&c0->user_state);
+	  c0->uid_gid_set = true;
 	}
-      else if (c->c2.uid_gid_specified)
+      else if (c0->uid_gid_specified)
 	{
 	  msg (M_INFO, "NOTE: UID/GID downgrade %s", why_not);
 	}
@@ -951,7 +961,6 @@ pull_permission_mask (void)
   return (  OPT_P_UP
 	  | OPT_P_ROUTE
 	  | OPT_P_IPWIN32
-	  | OPT_P_SETENV
 	  | OPT_P_SHAPER
 	  | OPT_P_TIMER
 	  | OPT_P_PERSIST
@@ -1018,7 +1027,7 @@ do_hold (struct context *c)
   if (management)
     {
       /* if c is defined, daemonize before hold */
-      if (c && c->options.daemon && management_would_hold (management))
+      if (c && c->options.daemon && management_should_daemonize (management))
 	do_init_first_time (c);
 
       /* block until management hold is released */
@@ -1804,25 +1813,30 @@ do_compute_occ_strings (struct context *c)
 static void
 do_init_first_time (struct context *c)
 {
-  if (c->first_time && !c->c2.did_we_daemonize)
+  if (c->first_time && !c->did_we_daemonize && !c->c0)
     {
+      struct context_0 *c0;
+
+      ALLOC_OBJ_CLEAR_GC (c->c0, struct context_0, &c->gc);
+      c0 = c->c0;
+      
       /* get user and/or group that we want to setuid/setgid to */
-      c->c2.uid_gid_specified =
-	get_group (c->options.groupname, &c->c2.group_state) |
-	get_user (c->options.username, &c->c2.user_state);
+      c0->uid_gid_specified =
+	get_group (c->options.groupname, &c0->group_state) |
+	get_user (c->options.username, &c0->user_state);
 
       /* get --writepid file descriptor */
-      get_pid_file (c->options.writepid, &c->c2.pid_state);
+      get_pid_file (c->options.writepid, &c0->pid_state);
 
       /* become a daemon if --daemon */
-      c->c2.did_we_daemonize = possibly_become_daemon (&c->options, c->first_time);
+      c->did_we_daemonize = possibly_become_daemon (&c->options, c->first_time);
 
       /* should we disable paging? */
-      if (c->options.mlock && c->c2.did_we_daemonize)
+      if (c->options.mlock && c->did_we_daemonize)
 	do_mlockall (true);	/* call again in case we daemonized */
 
       /* save process ID in a file */
-      write_pid (&c->c2.pid_state);
+      write_pid (&c0->pid_state);
 
       /* should we change scheduling priority? */
       set_nice (c->options.nice);
@@ -2284,6 +2298,12 @@ init_instance (struct context *c, const struct env_set *env, const unsigned int
 	goto sig;
     }
 
+#if P2MP
+  /* get passwords if undefined */
+  if (auth_retry_get () == AR_INTERACT)
+    init_query_passwords (c);
+#endif
+
   /* initialize context level 2 --verb/--mute parms */
   init_verb_mute (c, IVM_LEVEL_2);
 
@@ -2350,7 +2370,7 @@ init_instance (struct context *c, const struct env_set *env, const unsigned int
     else if (child)
       crypto_flags = CF_INIT_TLS_MULTI;
     do_init_crypto (c, crypto_flags);
-    if (IS_SIG (c))
+    if (IS_SIG (c) && !child)
       goto sig;
   }
 
@@ -2507,8 +2527,6 @@ inherit_context_child (struct context *dest,
       ASSERT (0);
     }
 
-  dest->first_time = false;
-
   dest->gc = gc_new ();
 
   ALLOC_OBJ_CLEAR_GC (dest->sig, struct signal_info, &dest->gc);
@@ -2542,7 +2560,7 @@ inherit_context_child (struct context *dest,
   dest->c1.plugins = src->c1.plugins;
 
   /* context init */
-  init_instance (dest, src->c2.es, CC_USR1_TO_HUP | CC_GC_FREE);
+  init_instance (dest, src->c2.es, CC_NO_CLOSE | CC_USR1_TO_HUP);
   if (IS_SIG (dest))
     return;
 
@@ -2584,6 +2602,7 @@ inherit_context_top (struct context *dest,
   dest->mode = CM_TOP_CLONE; 
 
   dest->first_time = false;
+  dest->c0 = NULL;
 
   options_detach (&dest->options);
   gc_detach (&dest->gc);
@@ -2610,6 +2629,9 @@ inherit_context_top (struct context *dest,
 void
 close_context (struct context *c, int sig, unsigned int flags)
 {
+  ASSERT (c);
+  ASSERT (c->sig);
+
   if (sig >= 0)
     c->sig->signal_received = sig;
 
@@ -2620,7 +2642,8 @@ close_context (struct context *c, int sig, unsigned int flags)
 	c->sig->signal_received = SIGHUP;
     }
 
-  close_instance (c);
+  if (!(flags & CC_NO_CLOSE))
+    close_instance (c);
 
   if (flags & CC_GC_FREE)
     context_gc_free (c);
