@@ -1564,7 +1564,7 @@ management_io (struct management *man)
 
 #endif
 
-inline bool
+static inline bool
 man_standalone_ok (const struct management *man)
 {
   return !man->settings.management_over_tunnel && man->connection.state != MS_INITIAL;
@@ -1796,6 +1796,16 @@ management_would_hold (struct management *man)
   return man->settings.hold && !man->persist.hold_release && man_standalone_ok (man);
 }
 
+/*
+ * Return true if (from the management interface's perspective) OpenVPN should
+ * daemonize.
+ */
+bool
+management_should_daemonize (struct management *man)
+{
+  return management_would_hold (man) || man->settings.up_query_passwords;
+}
+
 /*
  * If the hold flag is enabled, hibernate until a management client releases the hold.
  * Return true if the caller should not sleep for an additional time interval.
