@@ -5,12 +5,11 @@
  *             packet encryption, packet authentication, and
  *             packet compression.
  *
- *  Copyright (C) 2002-2004 James Yonan <jim@yonan.net>
+ *  Copyright (C) 2002-2005 OpenVPN Solutions LLC <info@openvpn.net>
  *
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  it under the terms of the GNU General Public License version 2
+ *  as published by the Free Software Foundation.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,6 +25,8 @@
 #ifndef MTU_H
 #define MTU_H
 
+#include "buffer.h"
+
 /*
  * 
  * Packet maninipulation routes such as encrypt, decrypt, compress, decompress
@@ -62,19 +63,29 @@
 /*
  * Default MTU of network over which tunnel data will pass by TCP/UDP.
  */
-#define LINK_MTU_DEFAULT   1300
+#define LINK_MTU_DEFAULT   1500
 
 /*
  * Default MTU of tunnel device.
  */
-#define TUN_MTU_DEFAULT    1300
+#define TUN_MTU_DEFAULT    1500
 
 /*
  * MTU Defaults for TAP devices
  */
-#define TAP_MTU_DEFAULT        ETHERNET_MTU
 #define TAP_MTU_EXTRA_DEFAULT  32
 
+/*
+ * Default MSSFIX value, used for reducing TCP MTU size
+ */
+#define MSSFIX_DEFAULT     1450
+
+/*
+ * Alignment of payload data such as IP packet or
+ * ethernet frame.
+ */
+#define PAYLOAD_ALIGN 4
+
 struct frame {
   /*
    * Maximum datagram size to be sent over the tunnel TCP/UDP channel.
@@ -83,7 +94,7 @@ struct frame {
   int link_mtu_dynamic;
 
   /*
-   * extra_frame: How many extra bytes might each subsystem (crypto, TLS, or, compression)
+   * How many extra bytes might each subsystem (crypto, TLS, or, compression)
    * add to frame in worst case?
    *
    * mtu + extra_frame = MTU of TCP/UDP transport
@@ -91,23 +102,33 @@ struct frame {
   int extra_frame;
 
   /*
-   * extra_buffer: Worst case size added to internal buffer due to functions
+   * Worst case size added to internal buffer due to functions
    * such as compression which can potentially expand the size of uncompressible
    * data.
    */
   int extra_buffer;
 
   /*
-   * extra_tun: max number of bytes in excess of tun mtu size that we might read
+   * Max number of bytes in excess of tun mtu size that we might read
    * or write from TUN/TAP device.
    */
   int extra_tun;
 
   /*
-   * extra_link: max number of bytes in excess on link mtu size that we might read
+   * Max number of bytes in excess of link mtu size that we might read
    * or write from UDP/TCP link.
    */
   int extra_link;
+
+  /*
+   * Alignment control
+   */
+# define FRAME_HEADROOM_MARKER_DECRYPT     (1<<0)
+# define FRAME_HEADROOM_MARKER_FRAGMENT    (1<<1)
+# define FRAME_HEADROOM_MARKER_READ_LINK   (1<<2)
+# define FRAME_HEADROOM_MARKER_READ_STREAM (1<<3)
+  unsigned int align_flags;
+  int align_adjust;
 };
 
 /* Routines which read struct frame should use the macros below */
@@ -154,17 +175,17 @@ struct frame {
 #define MAX_RW_SIZE_LINK(f)      (EXPANDED_SIZE(f) + (f)->extra_link)
 
 /*
- * In general, OpenVPN packet building routines set the initial
- * buffer store point this many bytes into the data buffer to
- * allow for efficient prepending.
+ * Control buffer headroom allocations to allow for efficient prepending.
  */
-#define FRAME_HEADROOM(f)        (TUN_LINK_DELTA(f) + (f)->extra_buffer + (f)->extra_link)
+#define FRAME_HEADROOM_BASE(f)     (TUN_LINK_DELTA(f) + (f)->extra_buffer + (f)->extra_link)
+#define FRAME_HEADROOM(f)          frame_headroom(f, 0)
+#define FRAME_HEADROOM_ADJ(f, fm)  frame_headroom(f, fm)
 
 /*
  * Max size of a buffer used to build a packet for output to
  * the TCP/UDP port.
  */
-#define BUF_SIZE(f)              (TUN_MTU_SIZE(f) + FRAME_HEADROOM(f) * 2)
+#define BUF_SIZE(f)              (TUN_MTU_SIZE(f) + FRAME_HEADROOM_BASE(f) * 2)
 
 /*
  * Function prototypes.
@@ -177,7 +198,10 @@ void frame_finalize (struct frame *frame,
 		     int tun_mtu);
 
 void frame_subtract_extra (struct frame *frame, const struct frame *src);
-void frame_print (const struct frame *frame, int level, const char *prefix);
+
+void frame_print (const struct frame *frame,
+		  int level,
+		  const char *prefix);
 
 void set_mtu_discover_type (int sd, int mtu_type);
 int translate_mtu_discover_type_name (const char *name);
@@ -191,6 +215,14 @@ int translate_mtu_discover_type_name (const char *name);
 
 void frame_set_mtu_dynamic (struct frame *frame, int mtu, unsigned int flags);
 
+/*
+ * allocate a buffer for socket or tun layer
+ */
+void alloc_buf_sock_tun (struct buffer *buf,
+			 const struct frame *frame,
+			 const bool tuntap_buffer,
+			 const unsigned int align_mask);
+
 /*
  * EXTENDED_SOCKET_ERROR_CAPABILITY functions -- print extra error info
  * on socket errors, such as PMTU size.  As of 2003.05.11, only works
@@ -200,38 +232,69 @@ void frame_set_mtu_dynamic (struct frame *frame, int mtu, unsigned int flags);
 #if EXTENDED_SOCKET_ERROR_CAPABILITY
 
 void set_sock_extended_error_passing (int sd);
-const char *format_extended_socket_error (int fd, int* mtu);
+const char *format_extended_socket_error (int fd, int *mtu, struct gc_arena *gc);
 
 #endif
 
 /*
- * Inline functions
+ * Calculate a starting offset into a buffer object, dealing with
+ * headroom and alignment issues.
+ */
+static inline int
+frame_headroom (const struct frame *f, const unsigned int flag_mask)
+{
+  const int offset = FRAME_HEADROOM_BASE (f);
+  const int adjust = (flag_mask & f->align_flags) ? f->align_adjust : 0;
+  const int delta = ((PAYLOAD_ALIGN << 24) - (offset + adjust)) & (PAYLOAD_ALIGN - 1);
+  return offset + delta;
+}
+
+/*
+ * frame member adjustment functions
  */
 
 static inline void
-frame_add_to_extra_frame (struct frame *frame, int increment)
+frame_add_to_extra_frame (struct frame *frame, const int increment)
 {
   frame->extra_frame += increment;
 }
 
 static inline void
-frame_add_to_extra_tun (struct frame *frame, int increment)
+frame_add_to_extra_tun (struct frame *frame, const int increment)
 {
   frame->extra_tun += increment;
 }
 
 static inline void
-frame_add_to_extra_link (struct frame *frame, int increment)
+frame_add_to_extra_link (struct frame *frame, const int increment)
 {
   frame->extra_link += increment;
 }
 
 static inline void
-frame_add_to_extra_buffer (struct frame *frame, int increment)
+frame_add_to_extra_buffer (struct frame *frame, const int increment)
 {
   frame->extra_buffer += increment;
 }
 
+static inline void
+frame_add_to_align_adjust (struct frame *frame, const int increment)
+{
+  frame->align_adjust += increment;
+}
+
+static inline void
+frame_align_to_extra_frame (struct frame *frame)
+{
+  frame->align_adjust = frame->extra_frame + frame->extra_link;
+}
+
+static inline void
+frame_or_align_flags (struct frame *frame, const unsigned int flag_mask)
+{
+  frame->align_flags |= flag_mask;
+}
+
 static inline bool
 frame_defined (const struct frame *frame)
 {
