@@ -95,6 +95,17 @@ typedef void *openvpn_plugin_handle_t;
  * function (such as tls-verify or auth-user-pass-verify), then
  * every module and script must return success (0) in order for
  * the connection to be authenticated.
+ *
+ * Notes:
+ *
+ * Plugins which use a privilege-separation model (by forking in
+ * their initialization function before the main OpenVPN process
+ * downgrades root privileges and/or executes a chroot) must
+ * daemonize after a fork if the "daemon" environmental variable is
+ * set.  In addition, if the "daemon_log_redirect" variable is set,
+ * the plugin should preserve stdout/stderr across the daemon()
+ * syscall.  See the daemonize() function in plugin/auth-pam/auth-pam.c
+ * for an example.
  */
 
 /*
@@ -103,6 +114,8 @@ typedef void *openvpn_plugin_handle_t;
 
 /*
  * FUNCTION: openvpn_plugin_open_v1
+ *
+ * REQUIRED: YES
  * 
  * Called on initial plug-in load.  OpenVPN will preserve plug-in state
  * across SIGUSR1 restarts but not across SIGHUP restarts.  A SIGHUP reset
@@ -140,6 +153,8 @@ OPENVPN_PLUGIN_DEF openvpn_plugin_handle_t OPENVPN_PLUGIN_FUNC(openvpn_plugin_op
  *
  * Called to perform the work of a given script type.
  *
+ * REQUIRED: YES
+ * 
  * ARGUMENTS
  *
  * handle : the openvpn_plugin_handle_t value which was returned by
@@ -166,6 +181,8 @@ OPENVPN_PLUGIN_DEF int OPENVPN_PLUGIN_FUNC(openvpn_plugin_func_v1)
 /*
  * FUNCTION: openvpn_plugin_close_v1
  *
+ * REQUIRED: YES
+ * 
  * ARGUMENTS
  *
  * handle : the openvpn_plugin_handle_t value which was returned by
@@ -175,3 +192,20 @@ OPENVPN_PLUGIN_DEF int OPENVPN_PLUGIN_FUNC(openvpn_plugin_func_v1)
  */
 OPENVPN_PLUGIN_DEF void OPENVPN_PLUGIN_FUNC(openvpn_plugin_close_v1)
      (openvpn_plugin_handle_t handle);
+
+/*
+ * FUNCTION: openvpn_plugin_abort_v1
+ *
+ * REQUIRED: NO
+ * 
+ * ARGUMENTS
+ *
+ * handle : the openvpn_plugin_handle_t value which was returned by
+ *          openvpn_plugin_open_v1.
+ *
+ * Called when OpenVPN is in the process of aborting due to a fatal error.
+ * Will only be called on an open context returned by a prior successful
+ * openvpn_plugin_open_v1 callback.
+ */
+OPENVPN_PLUGIN_DEF void OPENVPN_PLUGIN_FUNC(openvpn_plugin_abort_v1)
+     (openvpn_plugin_handle_t handle);
