@@ -102,6 +102,11 @@ main (int argc, char *argv[])
 {
   struct context c;
 
+#if PEDANTIC
+  fprintf (stderr, "Sorry, I was built with --enable-pedantic and I am incapable of doing any real work!\n");
+  return 1;
+#endif
+
   CLEAR (c);
 
   /* signify first time for components which can
