@@ -138,6 +138,33 @@ send_control (int fd, int code)
     return -1;
 }
 
+/*
+ * Daemonize if "daemon" env var is true.
+ * Preserve stderr across daemonization if
+ * "daemon_log_redirect" env var is true.
+ */
+static void
+daemonize (const char *envp[])
+{
+  const char *daemon_string = get_env ("daemon", envp);
+  if (daemon_string && daemon_string[0] == '1')
+    {
+      const char *log_redirect = get_env ("daemon_log_redirect", envp);
+      int fd = -1;
+      if (log_redirect && log_redirect[0] == '1')
+	fd = dup (2);
+      if (daemon (0, 0) < 0)
+	{
+	  fprintf (stderr, "DOWN-ROOT: daemonization failed\n");
+	}
+      else if (fd >= 3)
+	{
+	  dup2 (fd, 2);
+	  close (fd);
+	}
+    }
+}
+
 /*
  * Close most of parent's fds.
  * Keep stdin/stdout/stderr, plus one
@@ -348,6 +375,9 @@ openvpn_plugin_func_v1 (openvpn_plugin_handle_t handle, const int type, const ch
 	  /* Ignore most signals (the parent will receive them) */
 	  set_signals ();
 
+	  /* Daemonize if --daemon option is set. */
+	  daemonize (envp);
+
 	  /* execute the event loop */
 	  down_root_server (fd[1], context->command, argv, envp, context->verb);
 
@@ -399,6 +429,20 @@ openvpn_plugin_close_v1 (openvpn_plugin_handle_t handle)
   free_context (context);
 }
 
+OPENVPN_EXPORT void
+openvpn_plugin_abort_v1 (openvpn_plugin_handle_t handle)
+{
+  struct down_root_context *context = (struct down_root_context *) handle;
+
+  if (context->foreground_fd >= 0)
+    {
+      /* tell background process to exit */
+      send_control (context->foreground_fd, COMMAND_EXIT);
+      close (context->foreground_fd);
+      context->foreground_fd = -1;
+    }
+}
+
 /*
  * Background process -- runs with privilege.
  */
@@ -486,7 +530,7 @@ down_root_server (const int fd, char *command, const char *argv[], const char *e
 
 	case -1:
 	  fprintf (stderr, "DOWN-ROOT: BACKGROUND: read error on command channel\n");
-	  break;
+	  goto done;
 
 	default:
 	  fprintf (stderr, "DOWN-ROOT: BACKGROUND: unknown command code: code=%d, exiting\n",
