@@ -5,12 +5,11 @@
  *             packet encryption, packet authentication, and
  *             packet compression.
  *
- *  Copyright (C) 2002-2004 James Yonan <jim@yonan.net>
+ *  Copyright (C) 2002-2005 OpenVPN Solutions LLC <info@openvpn.net>
  *
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  it under the terms of the GNU General Public License version 2
+ *  as published by the Free Software Foundation.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -58,9 +57,9 @@ session_id_random (struct session_id *sid)
 }
 
 const char *
-session_id_print (const struct session_id *sid)
+session_id_print (const struct session_id *sid, struct gc_arena *gc)
 {
-  return format_hex (sid->id, SID_SIZE, 0);
+  return format_hex (sid->id, SID_SIZE, 0, gc);
 }
 
 #else
